#!/usr/bin/python
# -*- coding: utf-8 -*-

import pyjama_xml
import simplejson
import os

import functions

# Gettext - Übersetzung
functions.translation_gettext()
#def _(string):
#    return string


try:
    # pysqlite --> http://initd.org/tracker/pysqlite/wiki/pysqlite
    from pysqlite2 import dbapi2 as sqlite3
except ImportError:
    # Ab Python 2.5
    import sqlite3


home=functions.preparedirs()
db = os.path.join(home, "pyjama.db")

###################################################################
#
# create tables
# RETURNS: n/A
#
def create_tables():
    # Verbindung herstellen
    conn = sqlite3.connect(db)
    print _("Creating Tables")
    # Tabellen erstellen
    # ARTISTS

    sql = """
    CREATE TABLE artists (
      uid INTEGER PRIMARY KEY,
      id INTEGER,
      name TEXT,
      country TEXT,
      image TEXT,
      mbgid TEXT,
      state TEXT,
      city TEXT,
      url TEXT,
      latitude TEXT,
      longitude TEXT,
      albumcount INTEGER
    )
    """
    conn.execute(sql)
    conn.commit()

    # ALBUMS
    sql = """
    CREATE TABLE albums (
      uid INTEGER PRIMARY KEY,
      id INTEGER,
      name TEXT,
      url TEXT,
      releasedate TEXT,
      filename TEXT,
      id3genre INTEGER,
      mbgid TEXT,
      license_artwork TEXT,
      artist_id INTEGER,
      trackcount INTEGER
    )
    """
    conn.execute(sql)
    conn.commit()
    # TRACKS
    sql = """
    CREATE TABLE tracks (
      uid INTEGER PRIMARY KEY,
      id INTEGER,
      name TEXT,
      duration INTEGER,
      album_id INTEGER,
      artist_id INTEGER,
      numalbum INTEGER,
      filename TEXT,
      mbgid TEXT,
      id3genre INTEGER,
      license TEXT
    )
    """
    conn.execute(sql)
    conn.commit()
    
    conn.close()

###################################################################
#
# read out Jamendo's xml dump and fills the sql-database
# RETURNS: n/A
#
def create_db():
    databasegz = "dbdump_artistalbumtrack.xml.gz"
    database = "dbdump_artistalbumtrack.xml"
    
    print _("Downloading Database %s") % databasegz
    os.system("wget -O %s http://img.jamendo.com/data/dbdump_artistalbumtrack.xml.gz" % (os.path.join(home, databasegz)))
    print _("Unzipping DB %s") % databasegz
    os.system("gunzip %s" % os.path.join(home, databasegz)) # oder gzip -d

    conn = sqlite3.connect(db)

    print _("Reading XML- Database - this my take a while:")
    print _("Processing '%s'") % database
    artist, albums, tracks = pyjama_xml.parse_xml(os.path.join(home, database))

    # WERTE EINTRAGEN
    # ARTISTS
    sql = """
    INSERT INTO artists (
        id,
        name,
        country,
        image,
        mbgid,
        state,
        city,
        url,
        latitude,
        longitude,
        albumcount
    ) VALUES (
        :id,
        :name,
        :country,
        :image,
        :mbgid,
        :state,
        :city,
        :url,
        :latitude,
        :longitude,
        :albumcount
    )
    """
    print _("Inserting Artists")
    conn.executemany(sql, artist)
    conn.commit()
    
    #print "Converting Albums"
    #print "Converting some Objects to Strings"
    #for counter in xrange(len(albums)):
    #    albums[counter]['covers'] = simplejson.dumps(albums[counter]['covers'])
    #    albums[counter]['p2plinks'] = simplejson.dumps(albums[counter]['p2plinks'])
    # ALBUMS
    #conn.create_function('json', 1, txt)

    sql = """
    INSERT INTO albums (
        id,
        name,
        url,
        releasedate,
        filename,
        id3genre,
        mbgid,
        license_artwork,
        artist_id,
        trackcount
    ) VALUES (
        :id,
        :name,
        :url,
        :releasedate,
        :filename,
        :id3genre,
        :mbgid,
        :license_artwork,
        :artist_id,
        :trackcount
    )
    """
    print _("Inserting Albums")
    conn.executemany(sql, albums)
    conn.commit()

    # Tracks
    sql = """
    INSERT INTO tracks (
        id,
        name,
        duration,
        album_id,
        artist_id,
        numalbum,
        filename,
        mbgid,
        id3genre,
        license
    ) VALUES (
        :id,
        :name,
        :duration,
        :album_id,
        :artist_id,
        :numalbum,
        :filename,
        :mbgid,
        :id3genre,
        :license
    )
    """
    print _("Inserting Tracks")
    conn.executemany(sql, tracks)
    conn.commit()
    
    # Verbindung schliessen (nur zum Testen)
    conn.close()
    
    print _("Removing DB %s'") % database
    os.system("rm %s" % os.path.join(home, database))
    
    print _("Installation Finished")

###################################################################
#
# delete sqlite-database
# RETURNS: n/A
#
def delete_db():
    print _("removing old database...")
    try:
        os.remove(db)
    except:
        pass

#delete_db()
#create_tables()
#create_db()
        
