#!/usr/bin/python
# -*- coding: utf-8 -*-

import gettext
import os, sys
from constants import *

def install_dir():
    dirname = os.path.dirname(sys.argv[0])
    abspath =  os.path.abspath(dirname)

    if os.name == "nt":
        if os.path.exists(os.path.join(abspath, "clGstreamer010.py")):
            #libs here
            return abspath
        else:
            return os.path.join(os.getenv('PROGRAMFILES'), "pyjama")
    else:
        if os.path.exists(os.path.join(abspath, "clGstreamer010.py")):
            #libs here
            return abspath
        else:
            return INSTALL_DIR_POSIX

###################################################################
#
# Getting and making pyjama- dirs
# RETURNS: string (home-directory)
#
def preparedirs():
    home = os.getenv("HOME")
    if os.path.exists(os.path.join(home, ".pyjama")):
        os.system("chmod -R 0777 %s" % os.path.join(home, ".pyjama"))
    if os.path.exists(os.path.join(home, ".pyjama", "images")) == False:
        os.makedirs(os.path.join(home, ".pyjama", "images"))
    if os.path.exists(os.path.join(home, ".pyjama", "cache")) == False:
        os.makedirs(os.path.join(home, ".pyjama", "cache"))
    if os.path.exists(os.path.join(home, ".pyjama", "cache", "long")) == False:
        os.makedirs(os.path.join(home, ".pyjama", "cache", "long"))
    if os.path.exists(os.path.join(home, ".pyjama", "cache", "short")) == False:
        os.makedirs(os.path.join(home, ".pyjama", "cache", "short"))
    if os.path.exists(os.path.join(home, ".pyjama", "themes")) == False:
        os.makedirs(os.path.join(home, ".pyjama", "themes"))
    home = os.path.join(home, ".pyjama")
    os.chmod(home, 0777)
    return home

def get_pyjama_dir():
    return os.path.join(os.getenv("HOME"), ".pyjama")

###################################################################
#
# Decoding Display Names
# RETURNS: string
#
from htmlentitydefs import name2codepoint as n2cp
import re

def substitute_entity(match):
    ent = match.group(2)
    if match.group(1) == "#":
        return unichr(int(ent))
    else:
        cp = n2cp.get(ent)

        if cp:
            return unichr(cp)
        else:
            return match.group()

def decode_htmlentities(string):
    entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")
    return entity_re.subn(substitute_entity, string)[0]

###################################################################
#
# Convert numeric genre-id to genre names
# RETURNS: string
#
def id2genre(id):
    genres = [
    "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", 
    "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies", "Other", 
    "Pop", "R&amp;B", "Rap", "Reggae", "Rock", "Techno", "Industrial", 
    "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack", 
    "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion", 
    "Trance", "Classical", "Instrumental", "Acid", "House", "Game", 
    "Sound Clip", "Gospel", "Noise", "Alt. Rock", "Bass", "Soul", 
    "Punk", "Space", "Meditative", "Instrum. Pop", "Instrum. Rock", 
    "Ethnic", "Gothic", "Darkwave", "Techno-Indust.", "Electronic", 
    "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", 
    "Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle", 
    "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave", 
    "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", 
    "Polka", "Retro", "Musical", "Rock &amp; Roll", "Hard Rock", "Folk", 
    "Folk/Rock", "National Folk", "Swing", "Fusion", "Bebob", "Latin", 
    "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", 
    "Progress. Rock", "Psychadel. Rock", "Symphonic Rock", "Slow Rock", 
    "Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", 
    "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", 
    "Booty Bass", "Primus", "Porn Groove", "Satire", "Slow Jam", 
    "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad", 
    "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", 
    "A Capella", "Euro-House", "Dance Hall", "Goa", "Drum &amp; Bass", 
    "Club-House", "Hardcore", "Terror", "Indie", "BritPop", "Negerpunk", 
    "Polsk Punk", "Beat", "Christian Gangsta Rap", "Heavy Metal", 
    "Black Metal", "Crossover", "Contemporary Christian", "Christian Rock",
    "Merengue", "Salsa", "Thrash Metal", "Anime", "Jpop", "Synthpop" 
    ]
    if id == None:
        return "n/A"
    return genres[int(id)]

###################################################################
#
# gettext - _()
#
def translation_gettext():
    path = "locale"
    if not os.path.exists(path):
        path = os.path.join(install_dir(), "locale")
    try:
        trans = gettext.translation('pyjama', path)
    except IOError:
        trans = gettext.translation('pyjama', path, ["en"])
        print "No language file for you found - using english language file"
    trans.install()

###################################################################
#
# Checks if some necessary modules can be found
# RETURNS: int
#    
def check_modules():
    err = 0
    nf1, nf2 = False, False

    try:
        import pygtk
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "pygtk"

    try:
        import gtk
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "gtk"
        
    try:
        import gtk.glade
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "gtk.glade"
        
    try:
        import simplejson
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "simplejson"

    try:
        import gst
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "gst (gstreamer)"
        
    try:
        import xml.sax
    except ImportError:
        err += 1
        print _("Module '%s' not found") % "xml.sax"

    try:
        import sqlite3
    except ImportError:
        nf1 = True
        
    try:
        import pysqlite2
    except ImportError:
        nf2 = True
        if nf1 and nf2:
            err += 1
            print _("You need to have sqlite3 or pysqlite2 or libsqlite3-0 installed")

    if err == 0:
        print _("All modules found")
    else:
        print _("%s module(s) missing") % err

    return err

        
